@@ -5,8 +5,6 @@
  * \file circuitpadding_machines.c
  * \brief Circuit padding state machines
  *
- * \detail
- *
  * Introduce circuit padding machines that will be used by Tor circuits, as
  * specified by proposal 302 "Hiding onion service clients using padding".
  *
@@ -155,7 +153,6 @@ circpad_machine_relay_hide_intro_circuits(smartlist_t *machines_sl)
   relay_machine->name = "relay_ip_circ";
 
   relay_machine->conditions.state_mask = CIRCPAD_CIRC_OPENED;
-  relay_machine->target_hopnum = 2;
 
   /* This is a relay-side machine */
   relay_machine->is_origin_side = 0;
@@ -387,7 +384,6 @@ circpad_machine_relay_hide_rend_circuits(smartlist_t *machines_sl)
   /* Only pad after the circuit has been built and pad to the middle */
   relay_machine->conditions.min_hops = 2;
   relay_machine->conditions.state_mask = CIRCPAD_CIRC_OPENED;
-  relay_machine->target_hopnum = 2;
 
   /* This is a relay-side machine */
   relay_machine->is_origin_side = 0;
@@ -408,7 +404,7 @@ circpad_machine_relay_hide_rend_circuits(smartlist_t *machines_sl)
   /* OBFUSCATE_CIRC_SETUP -> END transition when we send our first
    * padding packet and/or hit the state length (the state length is 1). */
   relay_machine->states[CIRCPAD_STATE_OBFUSCATE_CIRC_SETUP].
-      next_state[CIRCPAD_EVENT_PADDING_RECV] = CIRCPAD_STATE_END;
+      next_state[CIRCPAD_EVENT_PADDING_SENT] = CIRCPAD_STATE_END;
   relay_machine->states[CIRCPAD_STATE_OBFUSCATE_CIRC_SETUP].
       next_state[CIRCPAD_EVENT_LENGTH_COUNT] = CIRCPAD_STATE_END;
 
