@@ -1,4 +1,4 @@
-/* Copyright (c) 2016-2019, The Tor Project, Inc. */
+/* Copyright (c) 2016-2020, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -12,7 +12,7 @@
 
 #include "core/or/or.h"
 #include "app/config/config.h"
-#include "app/config/confparse.h"
+#include "lib/confmgt/confmgt.h"
 #include "lib/crypt_ops/crypto_util.h"
 #include "feature/dirauth/dirvote.h"
 #include "feature/nodelist/networkstatus.h"
@@ -26,18 +26,18 @@
 
 #include "app/config/or_state_st.h"
 
-/* Default filename of the shared random state on disk. */
+/** Default filename of the shared random state on disk. */
 static const char default_fname[] = "sr-state";
 
-/* String representation of a protocol phase. */
+/** String representation of a protocol phase. */
 static const char *phase_str[] = { "unknown", "commit", "reveal" };
 
-/* Our shared random protocol state. There is only one possible state per
+/** Our shared random protocol state. There is only one possible state per
  * protocol run so this is the global state which is reset at every run once
  * the shared random value has been computed. */
 static sr_state_t *sr_state = NULL;
 
-/* Representation of our persistent state on disk. The sr_state above
+/** Representation of our persistent state on disk. The sr_state above
  * contains the data parsed from this state. When we save to disk, we
  * translate the sr_state to this sr_disk_state. */
 static sr_disk_state_t *sr_disk_state = NULL;
@@ -51,24 +51,17 @@ static const char dstate_cur_srv_key[] = "SharedRandCurrentValue";
  * members with CONF_CHECK_VAR_TYPE. */
 DUMMY_TYPECHECK_INSTANCE(sr_disk_state_t);
 
-/* These next two are duplicates or near-duplicates from config.c */
-#define VAR(name, conftype, member, initvalue)                              \
-  { name, CONFIG_TYPE_ ## conftype, offsetof(sr_disk_state_t, member),      \
-      initvalue CONF_TEST_MEMBERS(sr_disk_state_t, conftype, member) }
-/* As VAR, but the option name and member name are the same. */
-#define V(member, conftype, initvalue) \
+#define VAR(varname,conftype,member,initvalue)                          \
+  CONFIG_VAR_ETYPE(sr_disk_state_t, varname, conftype, member, 0, initvalue)
+#define V(member,conftype,initvalue)            \
   VAR(#member, conftype, member, initvalue)
-/* Our persistent state magic number. */
+
+/** Our persistent state magic number. */
 #define SR_DISK_STATE_MAGIC 0x98AB1254
 
-static int
-disk_state_validate_cb(void *old_state, void *state, void *default_state,
-                       int from_setconf, char **msg);
-static void disk_state_free_cb(void *);
-
-/* Array of variables that are saved to disk as a persistent state. */
-static config_var_t state_vars[] = {
-  V(Version,                    UINT, "0"),
+/** Array of variables that are saved to disk as a persistent state. */
+static const config_var_t state_vars[] = {
+  V(Version,                    POSINT, "0"),
   V(TorVersion,                 STRING, NULL),
   V(ValidAfter,                 ISOTIME, NULL),
   V(ValidUntil,                 ISOTIME, NULL),
@@ -81,28 +74,43 @@ static config_var_t state_vars[] = {
   END_OF_CONFIG_VARS
 };
 
-/* "Extra" variable in the state that receives lines we can't parse. This
+/** "Extra" variable in the state that receives lines we can't parse. This
  * lets us preserve options from versions of Tor newer than us. */
-static config_var_t state_extra_var = {
-  "__extra", CONFIG_TYPE_LINELIST,
-  offsetof(sr_disk_state_t, ExtraLines), NULL
-  CONF_TEST_MEMBERS(sr_disk_state_t, LINELIST, ExtraLines)
+static const struct_member_t state_extra_var = {
+  .name = "__extra",
+  .type = CONFIG_TYPE_LINELIST,
+  .offset = offsetof(sr_disk_state_t, ExtraLines),
 };
 
-/* Configuration format of sr_disk_state_t. */
+/** Configuration format of sr_disk_state_t. */
 static const config_format_t state_format = {
-  sizeof(sr_disk_state_t),
-  SR_DISK_STATE_MAGIC,
-  offsetof(sr_disk_state_t, magic_),
-  NULL,
-  NULL,
-  state_vars,
-  disk_state_validate_cb,
-  disk_state_free_cb,
-  &state_extra_var,
+  .size = sizeof(sr_disk_state_t),
+  .magic = {
+   "sr_disk_state_t",
+   SR_DISK_STATE_MAGIC,
+   offsetof(sr_disk_state_t, magic_),
+  },
+  .vars = state_vars,
+  .extra = &state_extra_var,
 };
 
-/* Return a string representation of a protocol phase. */
+/** Global configuration manager for the shared-random state file */
+static config_mgr_t *shared_random_state_mgr = NULL;
+
+/** Return the configuration manager for the shared-random state file. */
+static const config_mgr_t *
+get_srs_mgr(void)
+{
+  if (PREDICT_UNLIKELY(shared_random_state_mgr == NULL)) {
+    shared_random_state_mgr = config_mgr_new(&state_format);
+    config_mgr_freeze(shared_random_state_mgr);
+  }
+  return shared_random_state_mgr;
+}
+
+static void state_query_del_(sr_state_object_t obj_type, void *data);
+
+/** Return a string representation of a protocol phase. */
 STATIC const char *
 get_phase_str(sr_phase_t phase)
 {
@@ -120,7 +128,7 @@ get_phase_str(sr_phase_t phase)
 
   return the_string;
 }
-/* Return the time we should expire the state file created at <b>now</b>.
+/** Return the time we should expire the state file created at <b>now</b>.
  * We expire the state file in the beginning of the next protocol run. */
 STATIC time_t
 get_state_valid_until_time(time_t now)
@@ -151,7 +159,7 @@ get_state_valid_until_time(time_t now)
   return valid_until;
 }
 
-/* Given the consensus 'valid-after' time, return the protocol phase we should
+/** Given the consensus 'valid-after' time, return the protocol phase we should
  * be in. */
 STATIC sr_phase_t
 get_sr_protocol_phase(time_t valid_after)
@@ -171,7 +179,7 @@ get_sr_protocol_phase(time_t valid_after)
   }
 }
 
-/* Add the given <b>commit</b> to <b>state</b>. It MUST be a valid commit
+/** Add the given <b>commit</b> to <b>state</b>. It MUST be a valid commit
  * and there shouldn't be a commit from the same authority in the state
  * already else verification hasn't been done prior. This takes ownership of
  * the commit once in our state. */
@@ -196,7 +204,7 @@ commit_add_to_state(sr_commit_t *commit, sr_state_t *state)
   }
 }
 
-/* Helper: deallocate a commit object. (Used with digestmap_free(), which
+/** Helper: deallocate a commit object. (Used with digestmap_free(), which
  * requires a function pointer whose argument is void *). */
 static void
 commit_free_(void *p)
@@ -207,7 +215,7 @@ commit_free_(void *p)
 #define state_free(val) \
   FREE_AND_NULL(sr_state_t, state_free_, (val))
 
-/* Free a state that was allocated with state_new(). */
+/** Free a state that was allocated with state_new(). */
 static void
 state_free_(sr_state_t *state)
 {
@@ -221,7 +229,7 @@ state_free_(sr_state_t *state)
   tor_free(state);
 }
 
-/* Allocate an sr_state_t object and returns it. If no <b>fname</b>, the
+/** Allocate an sr_state_t object and returns it. If no <b>fname</b>, the
  * default file name is used. This function does NOT initialize the state
  * timestamp, phase or shared random value. NULL is never returned. */
 static sr_state_t *
@@ -240,7 +248,7 @@ state_new(const char *fname, time_t now)
   return new_state;
 }
 
-/* Set our global state pointer with the one given. */
+/** Set our global state pointer with the one given. */
 static void
 state_set(sr_state_t *state)
 {
@@ -254,34 +262,33 @@ state_set(sr_state_t *state)
 #define disk_state_free(val) \
   FREE_AND_NULL(sr_disk_state_t, disk_state_free_, (val))
 
-/* Free an allocated disk state. */
+/** Free an allocated disk state. */
 static void
 disk_state_free_(sr_disk_state_t *state)
 {
   if (state == NULL) {
     return;
   }
-  config_free(&state_format, state);
+  config_free(get_srs_mgr(), state);
 }
 
-/* Allocate a new disk state, initialize it and return it. */
+/** Allocate a new disk state, initialize it and return it. */
 static sr_disk_state_t *
 disk_state_new(time_t now)
 {
-  sr_disk_state_t *new_state = tor_malloc_zero(sizeof(*new_state));
+  sr_disk_state_t *new_state = config_new(get_srs_mgr());
 
-  new_state->magic_ = SR_DISK_STATE_MAGIC;
   new_state->Version = SR_PROTO_VERSION;
   new_state->TorVersion = tor_strdup(get_version());
   new_state->ValidUntil = get_state_valid_until_time(now);
   new_state->ValidAfter = now;
 
   /* Init config format. */
-  config_init(&state_format, new_state);
+  config_init(get_srs_mgr(), new_state);
   return new_state;
 }
 
-/* Set our global disk state with the given state. */
+/** Set our global disk state with the given state. */
 static void
 disk_state_set(sr_disk_state_t *state)
 {
@@ -292,7 +299,7 @@ disk_state_set(sr_disk_state_t *state)
   sr_disk_state = state;
 }
 
-/* Return -1 if the disk state is invalid (something in there that we can't or
+/** Return -1 if the disk state is invalid (something in there that we can't or
  * shouldn't use). Return 0 if everything checks out. */
 static int
 disk_state_validate(const sr_disk_state_t *state)
@@ -327,31 +334,7 @@ disk_state_validate(const sr_disk_state_t *state)
   return -1;
 }
 
-/* Validate the disk state (NOP for now). */
-static int
-disk_state_validate_cb(void *old_state, void *state, void *default_state,
-                       int from_setconf, char **msg)
-{
-  /* We don't use these; only options do. */
-  (void) from_setconf;
-  (void) default_state;
-  (void) old_state;
-
-  /* This is called by config_dump which is just before we are about to
-   * write it to disk. At that point, our global memory state has been
-   * copied to the disk state so it's fair to assume it's trustable. */
-  (void) state;
-  (void) msg;
-  return 0;
-}
-
-static void
-disk_state_free_cb(void *state)
-{
-  disk_state_free_(state);
-}
-
-/* Parse the Commit line(s) in the disk state and translate them to the
+/** Parse the Commit line(s) in the disk state and translate them to the
  * the memory state. Return 0 on success else -1 on error. */
 static int
 disk_state_parse_commits(sr_state_t *state,
@@ -406,7 +389,7 @@ disk_state_parse_commits(sr_state_t *state,
   return -1;
 }
 
-/* Parse a share random value line from the disk state and save it to dst
+/** Parse a share random value line from the disk state and save it to dst
  * which is an allocated srv object. Return 0 on success else -1. */
 static int
 disk_state_parse_srv(const char *value, sr_srv_t *dst)
@@ -441,7 +424,7 @@ disk_state_parse_srv(const char *value, sr_srv_t *dst)
   return ret;
 }
 
-/* Parse both SharedRandCurrentValue and SharedRandPreviousValue line from
+/** Parse both SharedRandCurrentValue and SharedRandPreviousValue line from
  * the state. Return 0 on success else -1. */
 static int
 disk_state_parse_sr_values(sr_state_t *state,
@@ -492,7 +475,7 @@ disk_state_parse_sr_values(sr_state_t *state,
   return -1;
 }
 
-/* Parse the given disk state and set a newly allocated state. On success,
+/** Parse the given disk state and set a newly allocated state. On success,
  * return that state else NULL. */
 static sr_state_t *
 disk_state_parse(const sr_disk_state_t *new_disk_state)
@@ -526,7 +509,7 @@ disk_state_parse(const sr_disk_state_t *new_disk_state)
   return NULL;
 }
 
-/* From a valid commit object and an allocated config line, set the line's
+/** From a valid commit object and an allocated config line, set the line's
  * value to the state string representation of a commit. */
 static void
 disk_state_put_commit_line(const sr_commit_t *commit, config_line_t *line)
@@ -536,7 +519,7 @@ disk_state_put_commit_line(const sr_commit_t *commit, config_line_t *line)
   tor_assert(commit);
   tor_assert(line);
 
-  if (!tor_mem_is_zero(commit->encoded_reveal,
+  if (!fast_mem_is_zero(commit->encoded_reveal,
                        sizeof(commit->encoded_reveal))) {
     /* Add extra whitespace so we can format the line correctly. */
     tor_asprintf(&reveal_str, " %s", commit->encoded_reveal);
@@ -553,7 +536,7 @@ disk_state_put_commit_line(const sr_commit_t *commit, config_line_t *line)
   }
 }
 
-/* From a valid srv object and an allocated config line, set the line's
+/** From a valid srv object and an allocated config line, set the line's
  * value to the state string representation of a shared random value. */
 static void
 disk_state_put_srv_line(const sr_srv_t *srv, config_line_t *line)
@@ -571,7 +554,7 @@ disk_state_put_srv_line(const sr_srv_t *srv, config_line_t *line)
   tor_asprintf(&line->value, "%" PRIu64 " %s", srv->num_reveals, encoded);
 }
 
-/* Reset disk state that is free allocated memory and zeroed the object. */
+/** Reset disk state that is free allocated memory and zeroed the object. */
 static void
 disk_state_reset(void)
 {
@@ -581,15 +564,16 @@ disk_state_reset(void)
   config_free_lines(sr_disk_state->ExtraLines);
   tor_free(sr_disk_state->TorVersion);
 
-  /* Clean up the struct */
-  memset(sr_disk_state, 0, sizeof(*sr_disk_state));
+  /* Clear other fields. */
+  sr_disk_state->ValidAfter = 0;
+  sr_disk_state->ValidUntil = 0;
+  sr_disk_state->Version = 0;
 
   /* Reset it with useful data */
-  sr_disk_state->magic_ = SR_DISK_STATE_MAGIC;
   sr_disk_state->TorVersion = tor_strdup(get_version());
 }
 
-/* Update our disk state based on our global SR state. */
+/** Update our disk state based on our global SR state. */
 static void
 disk_state_update(void)
 {
@@ -633,7 +617,7 @@ disk_state_update(void)
   } DIGESTMAP_FOREACH_END;
 }
 
-/* Load state from disk and put it into our disk state. If the state passes
+/** Load state from disk and put it into our disk state. If the state passes
  * validation, our global state will be updated with it. Return 0 on
  * success. On error, -EINVAL is returned if the state on disk did contained
  * something malformed or is unreadable. -ENOENT is returned indicating that
@@ -651,7 +635,7 @@ disk_state_load_from_disk(void)
   return ret;
 }
 
-/* Helper for disk_state_load_from_disk(). */
+/** Helper for disk_state_load_from_disk(). */
 STATIC int
 disk_state_load_from_disk_impl(const char *fname)
 {
@@ -680,7 +664,7 @@ disk_state_load_from_disk_impl(const char *fname)
     }
 
     disk_state = disk_state_new(time(NULL));
-    config_assign(&state_format, disk_state, lines, 0, &errmsg);
+    config_assign(get_srs_mgr(), disk_state, lines, 0, &errmsg);
     config_free_lines(lines);
     if (errmsg) {
       log_warn(LD_DIR, "SR: Reading state error: %s", errmsg);
@@ -713,7 +697,7 @@ disk_state_load_from_disk_impl(const char *fname)
   return ret;
 }
 
-/* Save the disk state to disk but before that update it from the current
+/** Save the disk state to disk but before that update it from the current
  * state so we always have the latest. Return 0 on success else -1. */
 static int
 disk_state_save_to_disk(void)
@@ -733,7 +717,7 @@ disk_state_save_to_disk(void)
   /* Make sure that our disk state is up to date with our memory state
    * before saving it to disk. */
   disk_state_update();
-  state = config_dump(&state_format, NULL, sr_disk_state, 0, 0);
+  state = config_dump(get_srs_mgr(), NULL, sr_disk_state, 0, 0);
   format_local_iso_time(tbuf, now);
   tor_asprintf(&content,
                "# Tor shared random state file last generated on %s "
@@ -757,7 +741,7 @@ disk_state_save_to_disk(void)
   return ret;
 }
 
-/* Reset our state to prepare for a new protocol run. Once this returns, all
+/** Reset our state to prepare for a new protocol run. Once this returns, all
  * commits in the state will be removed and freed. */
 STATIC void
 reset_state_for_new_protocol_run(time_t valid_after)
@@ -778,7 +762,7 @@ reset_state_for_new_protocol_run(time_t valid_after)
   sr_state_delete_commits();
 }
 
-/* This is the first round of the new protocol run starting at
+/** This is the first round of the new protocol run starting at
  * <b>valid_after</b>. Do the necessary housekeeping. */
 STATIC void
 new_protocol_run(time_t valid_after)
@@ -812,7 +796,7 @@ new_protocol_run(time_t valid_after)
   }
 }
 
-/* Return 1 iff the <b>next_phase</b> is a phase transition from the current
+/** Return 1 iff the <b>next_phase</b> is a phase transition from the current
  * phase that is it's different. */
 STATIC int
 is_phase_transition(sr_phase_t next_phase)
@@ -820,7 +804,7 @@ is_phase_transition(sr_phase_t next_phase)
   return sr_state->phase != next_phase;
 }
 
-/* Helper function: return a commit using the RSA fingerprint of the
+/** Helper function: return a commit using the RSA fingerprint of the
  * authority or NULL if no such commit is known. */
 static sr_commit_t *
 state_query_get_commit(const char *rsa_fpr)
@@ -829,11 +813,14 @@ state_query_get_commit(const char *rsa_fpr)
   return digestmap_get(sr_state->commits, rsa_fpr);
 }
 
-/* Helper function: This handles the GET state action using an
+/** Helper function: This handles the GET state action using an
  * <b>obj_type</b> and <b>data</b> needed for the action. */
 static void *
 state_query_get_(sr_state_object_t obj_type, const void *data)
 {
+  if (BUG(!sr_state))
+    return NULL;
+
   void *obj = NULL;
 
   switch (obj_type) {
@@ -861,24 +848,45 @@ state_query_get_(sr_state_object_t obj_type, const void *data)
   return obj;
 }
 
-/* Helper function: This handles the PUT state action using an
- * <b>obj_type</b> and <b>data</b> needed for the action. */
+/** Helper function: This handles the PUT state action using an
+ * <b>obj_type</b> and <b>data</b> needed for the action.
+ * PUT frees the previous data before replacing it, if needed. */
 static void
 state_query_put_(sr_state_object_t obj_type, void *data)
 {
+  if (BUG(!sr_state))
+    return;
+
   switch (obj_type) {
   case SR_STATE_OBJ_COMMIT:
   {
     sr_commit_t *commit = data;
     tor_assert(commit);
+    /* commit_add_to_state() frees the old commit, if there is one */
     commit_add_to_state(commit, sr_state);
     break;
   }
   case SR_STATE_OBJ_CURSRV:
-    sr_state->current_srv = (sr_srv_t *) data;
+      /* Check if the new pointer is the same as the old one: if it is, it's
+       * probably a bug. The caller may have confused current and previous,
+       * or they may have forgotten to sr_srv_dup().
+       * Putting NULL multiple times is allowed. */
+    if (!BUG(data && sr_state->current_srv == (sr_srv_t *) data)) {
+      /* We own the old SRV, so we need to free it.  */
+      state_query_del_(SR_STATE_OBJ_CURSRV, NULL);
+      sr_state->current_srv = (sr_srv_t *) data;
+    }
     break;
   case SR_STATE_OBJ_PREVSRV:
-    sr_state->previous_srv = (sr_srv_t *) data;
+      /* Check if the new pointer is the same as the old one: if it is, it's
+       * probably a bug. The caller may have confused current and previous,
+       * or they may have forgotten to sr_srv_dup().
+       * Putting NULL multiple times is allowed. */
+    if (!BUG(data && sr_state->previous_srv == (sr_srv_t *) data)) {
+      /* We own the old SRV, so we need to free it.  */
+      state_query_del_(SR_STATE_OBJ_PREVSRV, NULL);
+      sr_state->previous_srv = (sr_srv_t *) data;
+    }
     break;
   case SR_STATE_OBJ_VALID_AFTER:
     sr_state->valid_after = *((time_t *) data);
@@ -893,11 +901,14 @@ state_query_put_(sr_state_object_t obj_type, void *data)
   }
 }
 
-/* Helper function: This handles the DEL_ALL state action using an
+/** Helper function: This handles the DEL_ALL state action using an
  * <b>obj_type</b> and <b>data</b> needed for the action. */
 static void
 state_query_del_all_(sr_state_object_t obj_type)
 {
+  if (BUG(!sr_state))
+    return;
+
   switch (obj_type) {
   case SR_STATE_OBJ_COMMIT:
   {
@@ -908,7 +919,7 @@ state_query_del_all_(sr_state_object_t obj_type)
     } DIGESTMAP_FOREACH_END;
     break;
   }
-  /* The following object are _NOT_ suppose to be removed. */
+  /* The following objects are _NOT_ supposed to be removed. */
   case SR_STATE_OBJ_CURSRV:
   case SR_STATE_OBJ_PREVSRV:
   case SR_STATE_OBJ_PHASE:
@@ -919,13 +930,16 @@ state_query_del_all_(sr_state_object_t obj_type)
   }
 }
 
-/* Helper function: This handles the DEL state action using an
+/** Helper function: This handles the DEL state action using an
  * <b>obj_type</b> and <b>data</b> needed for the action. */
 static void
 state_query_del_(sr_state_object_t obj_type, void *data)
 {
   (void) data;
 
+  if (BUG(!sr_state))
+    return;
+
   switch (obj_type) {
   case SR_STATE_OBJ_PREVSRV:
     tor_free(sr_state->previous_srv);
@@ -942,7 +956,7 @@ state_query_del_(sr_state_object_t obj_type, void *data)
   }
 }
 
-/* Query state using an <b>action</b> for an object type <b>obj_type</b>.
+/** Query state using an <b>action</b> for an object type <b>obj_type</b>.
  * The <b>data</b> pointer needs to point to an object that the action needs
  * to use and if anything is required to be returned, it is stored in
  * <b>out</b>.
@@ -984,36 +998,36 @@ state_query(sr_state_action_t action, sr_state_object_t obj_type,
   }
 }
 
-/* Delete the current SRV value from the state freeing it and the value is set
+/** Delete the current SRV value from the state freeing it and the value is set
  * to NULL meaning empty. */
-static void
+STATIC void
 state_del_current_srv(void)
 {
   state_query(SR_STATE_ACTION_DEL, SR_STATE_OBJ_CURSRV, NULL, NULL);
 }
 
-/* Delete the previous SRV value from the state freeing it and the value is
+/** Delete the previous SRV value from the state freeing it and the value is
  * set to NULL meaning empty. */
-static void
+STATIC void
 state_del_previous_srv(void)
 {
   state_query(SR_STATE_ACTION_DEL, SR_STATE_OBJ_PREVSRV, NULL, NULL);
 }
 
-/* Rotate SRV value by freeing the previous value, assigning the current
- * value to the previous one and nullifying the current one. */
+/** Rotate SRV value by setting the previous SRV to the current SRV, and
+ * clearing the current SRV. */
 STATIC void
 state_rotate_srv(void)
 {
   /* First delete previous SRV from the state. Object will be freed. */
   state_del_previous_srv();
-  /* Set previous SRV with the current one. */
-  sr_state_set_previous_srv(sr_state_get_current_srv());
-  /* Nullify the current srv. */
+  /* Set previous SRV to a copy of the current one. */
+  sr_state_set_previous_srv(sr_srv_dup(sr_state_get_current_srv()));
+  /* Free and NULL the current srv. */
   sr_state_set_current_srv(NULL);
 }
 
-/* Set valid after time in the our state. */
+/** Set valid after time in the our state. */
 void
 sr_state_set_valid_after(time_t valid_after)
 {
@@ -1021,7 +1035,7 @@ sr_state_set_valid_after(time_t valid_after)
               (void *) &valid_after, NULL);
 }
 
-/* Return the phase we are currently in according to our state. */
+/** Return the phase we are currently in according to our state. */
 sr_phase_t
 sr_state_get_phase(void)
 {
@@ -1030,7 +1044,9 @@ sr_state_get_phase(void)
   return *(sr_phase_t *) ptr;
 }
 
-/* Return the previous SRV value from our state. Value CAN be NULL. */
+/** Return the previous SRV value from our state. Value CAN be NULL.
+ * The state object owns the SRV, so the calling code should not free the SRV.
+ * Use sr_srv_dup() if you want to keep a copy of the SRV. */
 const sr_srv_t *
 sr_state_get_previous_srv(void)
 {
@@ -1040,7 +1056,7 @@ sr_state_get_previous_srv(void)
   return srv;
 }
 
-/* Set the current SRV value from our state. Value CAN be NULL. The srv
+/** Set the current SRV value from our state. Value CAN be NULL. The srv
  * object ownership is transferred to the state object. */
 void
 sr_state_set_previous_srv(const sr_srv_t *srv)
@@ -1049,7 +1065,9 @@ sr_state_set_previous_srv(const sr_srv_t *srv)
               NULL);
 }
 
-/* Return the current SRV value from our state. Value CAN be NULL. */
+/** Return the current SRV value from our state. Value CAN be NULL.
+ * The state object owns the SRV, so the calling code should not free the SRV.
+ * Use sr_srv_dup() if you want to keep a copy of the SRV. */
 const sr_srv_t *
 sr_state_get_current_srv(void)
 {
@@ -1059,7 +1077,7 @@ sr_state_get_current_srv(void)
   return srv;
 }
 
-/* Set the current SRV value from our state. Value CAN be NULL. The srv
+/** Set the current SRV value from our state. Value CAN be NULL. The srv
  * object ownership is transferred to the state object. */
 void
 sr_state_set_current_srv(const sr_srv_t *srv)
@@ -1068,7 +1086,7 @@ sr_state_set_current_srv(const sr_srv_t *srv)
               NULL);
 }
 
-/* Clean all the SRVs in our state. */
+/** Clean all the SRVs in our state. */
 void
 sr_state_clean_srvs(void)
 {
@@ -1077,7 +1095,7 @@ sr_state_clean_srvs(void)
   state_del_current_srv();
 }
 
-/* Return a pointer to the commits map from our state. CANNOT be NULL. */
+/** Return a pointer to the commits map from our state. CANNOT be NULL. */
 digestmap_t *
 sr_state_get_commits(void)
 {
@@ -1088,7 +1106,7 @@ sr_state_get_commits(void)
   return commits;
 }
 
-/* Update the current SR state as needed for the upcoming voting round at
+/** Update the current SR state as needed for the upcoming voting round at
  * <b>valid_after</b>. */
 void
 sr_state_update(time_t valid_after)
@@ -1152,7 +1170,7 @@ sr_state_update(time_t valid_after)
   }
 }
 
-/* Return commit object from the given authority digest <b>rsa_identity</b>.
+/** Return commit object from the given authority digest <b>rsa_identity</b>.
  * Return NULL if not found. */
 sr_commit_t *
 sr_state_get_commit(const char *rsa_identity)
@@ -1166,7 +1184,7 @@ sr_state_get_commit(const char *rsa_identity)
   return commit;
 }
 
-/* Add <b>commit</b> to the permanent state. The commit object ownership is
+/** Add <b>commit</b> to the permanent state. The commit object ownership is
  * transferred to the state so the caller MUST not free it. */
 void
 sr_state_add_commit(sr_commit_t *commit)
@@ -1181,14 +1199,14 @@ sr_state_add_commit(sr_commit_t *commit)
             sr_commit_get_rsa_fpr(commit));
 }
 
-/* Remove all commits from our state. */
+/** Remove all commits from our state. */
 void
 sr_state_delete_commits(void)
 {
   state_query(SR_STATE_ACTION_DEL_ALL, SR_STATE_OBJ_COMMIT, NULL, NULL);
 }
 
-/* Copy the reveal information from <b>commit</b> into <b>saved_commit</b>.
+/** Copy the reveal information from <b>commit</b> into <b>saved_commit</b>.
  * This <b>saved_commit</b> MUST come from our current SR state. Once modified,
  * the disk state is updated. */
 void
@@ -1209,7 +1227,7 @@ sr_state_copy_reveal_info(sr_commit_t *saved_commit, const sr_commit_t *commit)
             sr_commit_get_rsa_fpr(saved_commit));
 }
 
-/* Set the fresh SRV flag from our state. This doesn't need to trigger a
+/** Set the fresh SRV flag from our state. This doesn't need to trigger a
  * disk state synchronization so we directly change the state. */
 void
 sr_state_set_fresh_srv(void)
@@ -1217,7 +1235,7 @@ sr_state_set_fresh_srv(void)
   sr_state->is_srv_fresh = 1;
 }
 
-/* Unset the fresh SRV flag from our state. This doesn't need to trigger a
+/** Unset the fresh SRV flag from our state. This doesn't need to trigger a
  * disk state synchronization so we directly change the state. */
 void
 sr_state_unset_fresh_srv(void)
@@ -1225,14 +1243,14 @@ sr_state_unset_fresh_srv(void)
   sr_state->is_srv_fresh = 0;
 }
 
-/* Return the value of the fresh SRV flag. */
+/** Return the value of the fresh SRV flag. */
 unsigned int
 sr_state_srv_is_fresh(void)
 {
   return sr_state->is_srv_fresh;
 }
 
-/* Cleanup and free our disk and memory state. */
+/** Cleanup and free our disk and memory state. */
 void
 sr_state_free_all(void)
 {
@@ -1241,9 +1259,10 @@ sr_state_free_all(void)
   /* Nullify our global state. */
   sr_state = NULL;
   sr_disk_state = NULL;
+  config_mgr_free(shared_random_state_mgr);
 }
 
-/* Save our current state in memory to disk. */
+/** Save our current state in memory to disk. */
 void
 sr_state_save(void)
 {
@@ -1251,7 +1270,7 @@ sr_state_save(void)
   state_query(SR_STATE_ACTION_SAVE, 0, NULL, NULL);
 }
 
-/* Return 1 iff the state has been initialized that is it exists in memory.
+/** Return 1 iff the state has been initialized that is it exists in memory.
  * Return 0 otherwise. */
 int
 sr_state_is_initialized(void)
@@ -1259,7 +1278,7 @@ sr_state_is_initialized(void)
   return sr_state == NULL ? 0 : 1;
 }
 
-/* Initialize the disk and memory state.
+/** Initialize the disk and memory state.
  *
  * If save_to_disk is set to 1, the state is immediately saved to disk after
  * creation else it's not thus only kept in memory.
@@ -1322,7 +1341,7 @@ sr_state_init(int save_to_disk, int read_from_disk)
 
 #ifdef TOR_UNIT_TESTS
 
-/* Set the current phase of the protocol. Used only by unit tests. */
+/** Set the current phase of the protocol. Used only by unit tests. */
 void
 set_sr_phase(sr_phase_t phase)
 {
@@ -1331,7 +1350,7 @@ set_sr_phase(sr_phase_t phase)
   sr_state->phase = phase;
 }
 
-/* Get the SR state. Used only by unit tests */
+/** Get the SR state. Used only by unit tests */
 sr_state_t *
 get_sr_state(void)
 {
